<?php
/**
*
* @package acp
* @version $Id: acp_cash.php 54 2008-07-13 20:25:23Z Handyman $
* @copyright (c) 2008 StarTrekGuide Group
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
* @package module_install
*/
class acp_cash_info
{
	var $version = '1.0.0b1';

	function module()
	{
		$action = request_var('action', '');

		if ($action == 'add' || $action == 'quickadd')
		{
			$this->install();
		}

		return array(
			'filename'	=> 'acp_cash',
			'title'		=> 'ACP_CASH',
			'version'	=> $this->version,
			'modes'		=> array(
				'bank'		=> array('title' => 'ACP_BANK', 'auth' => 'acl_a_bank_manager', 'cat' => 'ACP_CASH_MOD'),
				'settings'	=> array('title' => 'ACP_CASH_SETTINGS', 'auth' => 'acl_a_cash_manage', 'cat' => 'ACP_CASH_MOD'),
				'users'		=> array('title' => 'ACP_CASH_USERS', 'auth' => 'acl_a_cash_manage', 'cat' => 'ACP_CASH_MOD'),
			),
		);
	}

	function install()
	{
		global $phpbb_root_path, $phpEx, $db, $table_prefix, $config, $dbms;

		//make sure we don't already have this version installed
		if (isset($config['cash_version']))
		{
			if (!version_compare($config['cash_version'], $this->version, '<'))
			{
				return;
			}
			else
			{
				$this->upgrade();
				return;
			}
		}

		if (!class_exists('create_dbms'))
		{
			include($phpbb_root_path . 'includes/db/create_schema.' . $phpEx);
		}

		$create_dbms = new create_dbms();

		// If mysql is chosen, we need to adjust the schema filename slightly to reflect the correct version. ;)
		if ($dbms == 'mysql')
		{
			if (version_compare($db->mysql_version, '4.1.3', '>='))
			{
				$dbms_type = 'mysql_41';
			}
			else
			{
				$dbms_type = 'mysql_40';
			}
		}
		else if ($dbms == 'mysqli')
		{
			$dbms_type = 'mysql_41';
		}
		else
		{
			$dbms_type = $dbms;
		}


		$schema = "
			CREATE TABLE {$table_prefix}cash_amt (
				user_id mediumint(8) unsigned NOT NULL default '0',
				cash_id mediumint(8) unsigned NOT NULL default '1',
				cash_amt int(15) unsigned NOT NULL default '0',
				KEY cash_user (user_id, cash_id)
			);";

		$table_ary = $create_dbms->create_schema($schema);
		$db->sql_query($table_ary[$dbms_type]);

		$schema = "
			CREATE TABLE {$table_prefix}cash (
				cash_id MEDIUMINT(8) UNSIGNED NOT NULL AUTO_INCREMENT,
				cash_name VARCHAR(255) NOT NULL,
				cash_donate TINYINT(1) UNSIGNED NOT NULL DEFAULT '1',
				cash_exchange TINYINT(1) UNSIGNED NOT NULL DEFAULT '1',
				PRIMARY KEY cash_id (cash_id)
			);";

		$table_ary = $create_dbms->create_schema($schema);
		$db->sql_query($table_ary[$dbms_type]);

		$schema = "
			CREATE TABLE {$table_prefix}bank_amt (
				user_id mediumint(8) unsigned NOT NULL default '0',
				cash_id mediumint(8) unsigned NOT NULL default '1',
				cash_amt int(15) unsigned NOT NULL default '0',
				KEY cash_user (user_id, cash_id)
			);";

		$table_ary = $create_dbms->create_schema($schema);
		$db->sql_query($table_ary[$dbms_type]);

		$schema = "
			CREATE TABLE {$table_prefix}cash_rates (
				cash_id mediumint(8) unsigned NOT NULL default '0',
				exchange_rate decimal(5,2) unsigned NOT NULL default '1.00',
				exchange_fee decimal(5,2) unsigned NOT NULL default '0.00',
				exchange_percent decimal(5,2) unsigned NOT NULL default '0.00',
			);";

		$table_ary = $create_dbms->create_schema($schema);
		$db->sql_query($table_ary[$dbms_type]);

		add_log('admin', 'LOG_MODULE_ADD', 'Added Cash MOD database tables');

		set_config('cash_display_currency', 1);
		set_config('cash_display_name', 'Cash');
		set_config('cash_topic_per_word', 0);
		set_config('cash_topic_per_character', 0);
		set_config('cash_topic', 1);
		set_config('cash_topic_currency', 1);
		set_config('cash_post_per_word', 0);
		set_config('cash_post_per_character', 0);
		set_config('cash_post', 1);
		set_config('cash_post_currency', 1);
		set_config('cash_poll_per_option', 0);
		set_config('cash_poll', 0);
		set_config('cash_poll_currency', 1);
		set_config('cash_attachment_per_file', 0);
		set_config('cash_attachment', 0);
		set_config('cash_attachment_currency', 1);
		set_config('cash_version', '1.0.0b1');

		// insert Cash MOD permissions
		$permission_ary = array(
			'a_cash_manage'				=> $this->install_permissions('a_cash_manage', true, false),
			'a_bank_manager'			=> $this->install_permissions('a_bank_manager', true, false),
			'm_cash_manage'				=> $this->install_permissions('m_cash_manage', true, false),
			'm_bank_manager'			=> $this->install_permissions('m_bank_manager', true, false),
			'f_cash_topic_words'		=> $this->install_permissions('f_cash_topic_words', false, true),
			'f_cash_topic_characters'	=> $this->install_permissions('f_cash_topic_characters', false, true),
			'f_cash_topic'				=> $this->install_permissions('f_cash_topic', false, true),
			'f_cash_post_words'			=> $this->install_permissions('f_cash_post_words', false, true),
			'f_cash_post_characters'	=> $this->install_permissions('f_cash_post_characters', false, true),
			'f_cash_post'				=> $this->install_permissions('f_cash_post', false, true),
			'f_cash_poll_options'		=> $this->install_permissions('f_cash_poll_options', false, true),
			'f_cash_poll'				=> $this->install_permissions('f_cash_poll', false, true),
			'f_cash_attachment_files'	=> $this->install_permissions('f_cash_attachment_files', false, true),
			'f_cash_attachment'			=> $this->install_permissions('f_cash_attachment', false, true),
		);

		$this->install_modules();
		$this->install_forum_permissions($permission_ary['f_cash_post']);
		$this->install_forum_permissions($permission_ary['f_cash_topic']);
		$this->install_user_permissions($permission_ary);
		$this->populate_tables();

		global $cache, $auth;
		$cache->purge();

		// Clear permissions
		$auth->acl_clear_prefetch();
		cache_moderators();

		add_log('admin', 'LOG_PURGE_CACHE');

		trigger_error('Installation complete');
	}

	function uninstall()
	{
	}

	function install_modules()
	{
		global $db;

		// see if the Module category exists
		$sql = 'SELECT *
			FROM ' . MODULES_TABLE . "
			WHERE module_langname = 'ACP_CASH_MOD'
				AND module_class = 'acp'";
		$result = $db->sql_query($sql);
		$row = $db->sql_fetchrow($result);
		$db->sql_freeresult($result);

		if (!$row)
		{
			add_log('admin', 'LOG_MODULE_ADD', 'add ACP_CASH_MOD cat');
			$sql = 'SELECT *
				FROM ' . MODULES_TABLE . "
				WHERE module_langname = 'ACP_CAT_DOT_MODS'
					AND module_class = 'acp'";
			$result = $db->sql_query($sql);
			$row = $db->sql_fetchrow($result);
			$db->sql_freeresult($result);

			$sql_ary = array(
				'module_enabled'	=> 1,
				'module_display'	=> 1,
				'module_basename'	=> '',
				'module_class'		=> 'acp',
				'parent_id'			=> $row['module_id'],
				'left_id'			=> $row['right_id'],
				'right_id'			=> $row['right_id'] + 3,
				'module_langname'	=> 'ACP_CASH_MOD',
				'module_mode'		=> '',
				'module_auth'		=> '',
			);

			$sql = 'INSERT INTO ' . MODULES_TABLE . ' ' . $db->sql_build_array('INSERT', $sql_ary);
			$db->sql_query($sql);
			$module_id = $db->sql_nextid();
			add_log('admin', 'LOG_MODULE_ADD', 'insert module ' . $module_id);

			$sql = 'UPDATE ' . MODULES_TABLE . '
				SET left_id = left_id + 4,
					right_id = right_id + 4
				WHERE left_id >= ' . $sql_ary['left_id'] . '
					AND module_id != ' . $module_id;
			$db->sql_query($sql);

			$sql = 'UPDATE ' . MODULES_TABLE . '
				SET right_id = right_id + 4
				WHERE left_id < ' . $sql_ary['left_id'] . '
					AND right_id >= ' . $sql_ary['left_id'] . '
					AND module_id != ' . $module_id;
			$db->sql_query($sql);
			add_log('admin', 'LOG_MODULE_ADD', 'update module left and right id');
		}

		$this->quickadd('cash::users', $module_id, 'acp', false);
		$this->quickadd('cash::settings', $module_id, 'acp');
		$this->quickadd('cash::bank', $module_id, 'acp', false);
		$this->install_ucp_modules();

		add_log('admin', 'LOG_MODULE_ADD', 'Installed Cash MOD under the .MODS tab');
	}

	function install_ucp_modules()
	{
		global $db;

		$sql = 'SELECT MAX(right_id) as right_id
			FROM ' . MODULES_TABLE . "
			WHERE module_class = 'ucp'";
		$result = $db->sql_query($sql);
		$row = $db->sql_fetchrow($result);
		$db->sql_freeresult($result);

		$sql_ary = array(
			'module_enabled'	=> 1,
			'module_display'	=> 1,
			'module_basename'	=> '',
			'module_class'		=> 'ucp',
			'parent_id'			=> 0,
			'left_id'			=> $row['right_id'] + 1,
			'right_id'			=> $row['right_id'] + 2,
			'module_langname'	=> 'UCP_BANK',
			'module_mode'		=> '',
			'module_auth'		=> '',
		);
		$sql = 'INSERT INTO ' . MODULES_TABLE . ' ' . $db->sql_build_array('INSERT', $sql_ary);
		$db->sql_query($sql);

		$module_id = $db->sql_nextid();
		$this->quickadd('bank::donate', $module_id, 'ucp');

		add_log('admin', 'LOG_MODULE_ADD', 'Added Bank into the UCP');
	}

	function install_permissions($auth_option, $is_global, $is_local)
	{
		global $db;

		$sql_ary = array(
			'auth_option'	=> $auth_option,
			'is_global'		=> $is_global,
			'is_local'		=> $is_local,
		);

		$sql = 'INSERT INTO ' . ACL_OPTIONS_TABLE . ' ' . $db->sql_build_array('INSERT', $sql_ary);
		$db->sql_query($sql);

		return $db->sql_nextid();
	}

	/**
	 * Install posting permissions
	 *
	 * @param int $auth_option
	 */
	function install_forum_permissions($auth_option)
	{
		global $db;

		//setup cash to add up for all roles which allow posting without mod intervention
		$role_ary = array(
			'ROLE_FORUM_LIMITED',
			'ROLE_FORUM_LIMITED_POLLS',
			'ROLE_FORUM_STANDARD',
			'ROLE_FORUM_POLLS',
			'ROLE_FORUM_FULL'
		);

		$sql = 'SELECT role_id
			FROM ' . ACL_ROLES_TABLE . '
			WHERE ' . $db->sql_in_set('role_name', $role_ary) . "
			AND role_type = 'f_'";
		$result = $db->sql_query($sql);

		while ($row = $db->sql_fetchrow($result))
		{
			$sql_ary[] = array(
				'role_id'			=> $row['role_id'],
				'auth_option_id'	=> $auth_option,
				'auth_setting'		=> true,
			);
		}
		$db->sql_freeresult($result);
		$db->sql_multi_insert(ACL_ROLES_DATA_TABLE, $sql_ary);
	}

	/**
	 * Install user permissions for admins and moderators to be able to manage cash amounts and the bank.
	 *
	 * @param array $array
	 */
	function install_user_permissions($array)
	{
		global $db;

		$role_ary = array(
			'ROLE_MOD_FULL',
			'ROLE_ADMIN_STANDARD',
			'ROLE_ADMIN_FULL',
			'ROLE_ADMIN_USERGROUP',
		);

		$sql = 'SELECT role_id
			FROM ' . ACL_ROLES_TABLE . '
			WHERE ' . $db->sql_in_set('role_name', $role_ary);
		$result = $db->sql_query($sql);

		while ($row = $db->sql_fetchrow($result))
		{
			$sql_ary[] = array(
				'role_id'			=> $row['role_id'],
				'auth_option_id'	=> $array['a_cash_manage'],
				'auth_setting'		=> true,
			);

			$sql_ary[] = array(
				'role_id'			=> $row['role_id'],
				'auth_option_id'	=> $array['m_cash_manage'],
				'auth_setting'		=> true,
			);

			$sql_ary[] = array(
				'role_id'			=> $row['role_id'],
				'auth_option_id'	=> $array['a_bank_manager'],
				'auth_setting'		=> true,
			);

			$sql_ary[] = array(
				'role_id'			=> $row['role_id'],
				'auth_option_id'	=> $array['m_bank_manager'],
				'auth_setting'		=> true,
			);
		}
		$db->sql_freeresult($result);
		$db->sql_multi_insert(ACL_ROLES_DATA_TABLE, $sql_ary);
	}

	function upgrade()
	{
		//
	}

	function quickadd($quick_install, $parent_id, $module_class, $enabled = true)
	{
		if (!$quick_install || strpos($quick_install, '::') === false)
		{
			break;
		}

		list($module_basename, $module_mode) = explode('::', $quick_install);

		// Check if module name and mode exist...
		$fileinfo = $this->get_module_infos($module_basename, $module_class);
		$fileinfo = $fileinfo[$module_basename];

		if (isset($fileinfo['modes'][$module_mode]))
		{
			$module_data = array(
				'module_basename'	=> $module_basename,
				'module_enabled'	=> $enabled,
				'module_display'	=> (isset($fileinfo['modes'][$module_mode]['display'])) ? $fileinfo['modes'][$module_mode]['display'] : 1,
				'parent_id'			=> $parent_id,
				'module_class'		=> $module_class,
				'module_langname'	=> $fileinfo['modes'][$module_mode]['title'],
				'module_mode'		=> $module_mode,
				'module_auth'		=> $fileinfo['modes'][$module_mode]['auth'],
			);

			$errors = $this->update_module_data($module_data);

			if (!sizeof($errors))
			{
				$this->remove_cache_file();
			}
		}
	}

	/**
	* Get available module information from module files
	*/
	function get_module_infos($module = '', $module_class = false)
	{
		global $phpbb_root_path, $phpEx;

		$module_class = ($module_class === false) ? $this->module_class : $module_class;

		$directory = $phpbb_root_path . 'includes/' . $module_class . '/info/';
		$fileinfo = array();

		if (!$module)
		{
			$dh = @opendir($directory);

			if (!$dh)
			{
				return $fileinfo;
			}

			while (($file = readdir($dh)) !== false)
			{
				// Is module?
				if (preg_match('/^' . $module_class . '_.+\.' . $phpEx . '$/', $file))
				{
					$class = str_replace(".$phpEx", '', $file) . '_info';

					if (!class_exists($class))
					{
						include($directory . $file);
					}

					// Get module title tag
					if (class_exists($class))
					{
						$c_class = new $class();
						$module_info = $c_class->module();
						$fileinfo[str_replace($module_class . '_', '', $module_info['filename'])] = $module_info;
					}
				}
			}
			closedir($dh);

			ksort($fileinfo);
		}
		else
		{
			$filename = $module_class . '_' . basename($module);
			$class = $module_class . '_' . basename($module) . '_info';

			if (!class_exists($class))
			{
				include($directory . $filename . '.' . $phpEx);
			}

			// Get module title tag
			if (class_exists($class))
			{
				$c_class = new $class();
				$module_info = $c_class->module();
				$fileinfo[str_replace($module_class . '_', '', $module_info['filename'])] = $module_info;
			}
		}

		return $fileinfo;
	}

	function update_module_data(&$module_data, $run_inline = false)
	{
		global $db, $user;

		if (!isset($module_data['module_id']))
		{
			// no module_id means we're creating a new category/module
			if ($module_data['parent_id'])
			{
				$sql = 'SELECT left_id, right_id
					FROM ' . MODULES_TABLE . "
					WHERE module_class = '" . $db->sql_escape($module_data['module_class']) . "'
						AND module_id = " . (int) $module_data['parent_id'];
				$result = $db->sql_query($sql);
				$row = $db->sql_fetchrow($result);
				$db->sql_freeresult($result);

				if (!$row)
				{
					if ($run_inline)
					{
						return 'PARENT_NO_EXIST';
					}

					trigger_error($user->lang['PARENT_NO_EXIST'] . adm_back_link($this->u_action . '&amp;parent_id=' . $this->parent_id), E_USER_WARNING);
				}

				// Workaround
				$row['left_id'] = (int) $row['left_id'];
				$row['right_id'] = (int) $row['right_id'];

				$sql = 'UPDATE ' . MODULES_TABLE . "
					SET left_id = left_id + 2, right_id = right_id + 2
					WHERE module_class = '" . $db->sql_escape($module_data['module_class']) . "'
						AND left_id > {$row['right_id']}";
				$db->sql_query($sql);

				$sql = 'UPDATE ' . MODULES_TABLE . "
					SET right_id = right_id + 2
					WHERE module_class = '" . $db->sql_escape($module_data['module_class']) . "'
						AND {$row['left_id']} BETWEEN left_id AND right_id";
				$db->sql_query($sql);

				$module_data['left_id'] = (int) $row['right_id'];
				$module_data['right_id'] = (int) $row['right_id'] + 1;
			}
			else
			{
				$sql = 'SELECT MAX(right_id) AS right_id
					FROM ' . MODULES_TABLE . "
					WHERE module_class = '" . $db->sql_escape($module_data['module_class']) . "'";
				$result = $db->sql_query($sql);
				$row = $db->sql_fetchrow($result);
				$db->sql_freeresult($result);

				$module_data['left_id'] = (int) $row['right_id'] + 1;
				$module_data['right_id'] = (int) $row['right_id'] + 2;
			}

			$sql = 'INSERT INTO ' . MODULES_TABLE . ' ' . $db->sql_build_array('INSERT', $module_data);
			$db->sql_query($sql);

			$module_data['module_id'] = $db->sql_nextid();

			if (!$run_inline)
			{
				add_log('admin', 'LOG_MODULE_ADD', $module_data['module_langname']);
			}
		}
		else
		{
			$row = $this->get_module_row($module_data['module_id']);

			if ($module_data['module_basename'] && !$row['module_basename'])
			{
				// we're turning a category into a module
				$branch = $this->get_module_branch($module_data['module_id'], 'children', 'descending', false);

				if (sizeof($branch))
				{
					return array($user->lang['NO_CATEGORY_TO_MODULE']);
				}
			}

			if ($row['parent_id'] != $module_data['parent_id'])
			{
				$this->move_module($module_data['module_id'], $module_data['parent_id']);
			}

			$update_ary = $module_data;
			unset($update_ary['module_id']);

			$sql = 'UPDATE ' . MODULES_TABLE . '
				SET ' . $db->sql_build_array('UPDATE', $update_ary) . "
				WHERE module_class = '" . $db->sql_escape($module_data['module_class']) . "'
					AND module_id = " . (int) $module_data['module_id'];
			$db->sql_query($sql);

			if (!$run_inline)
			{
				add_log('admin', 'LOG_MODULE_EDIT', $this->lang_name($module_data['module_langname']));
			}
		}

		return array();
	}

	function populate_tables()
	{
		global $db;

		$sql_ary = array(
		   'cash_name'		=> 'Cash',
		   'cash_exchange'	=> true,
		   'cash_donate'	=> true,
		);
		$sql = 'INSERT INTO ' . CASH_TABLE . ' ' . $db->sql_build_array('INSERT', $sql_ary);
		$db->sql_query($sql);

		$sql_ary = array(
		   'cash_id'			=> $db->sql_nextid(),
		   'exchange_rate'		=> 1.00,
		   'exchange_fee'		=> 3.00,
		   'exchange_percent'	=> 2.00,
		);
		$sql = 'INSERT INTO ' . CASH_RATES_TABLE . ' ' . $db->sql_build_array('INSERT', $sql_ary);
		$db->sql_query($sql);
	}

	/**
	* Remove modules cache file
	*/
	function remove_cache_file()
	{
		global $cache;

		// Sanitise for future path use, it's escaped as appropriate for queries
		$p_class = str_replace(array('.', '/', '\\'), '', basename('acp'));

		$cache->destroy('_modules_' . $p_class);

		// Additionally remove sql cache
		$cache->destroy('sql', MODULES_TABLE);
	}
}