<?php
/**
*
* @package phpBB3
* @version $Id: cash_class.php 50 2008-07-06 21:19:04Z Handyman $
* @copyright (c) 2008 StarTrekGuide.com
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
* @ignore
*/
if (!defined('IN_PHPBB'))
{
	exit;
}

define('CASH_AMOUNT_TABLE',			$table_prefix . 'cash_amt');
define('CASH_RATES_TABLE',			$table_prefix . 'cash_rates');
define('CASH_TABLE',				$table_prefix . 'cash');

$cash = new cash();
$bank = new bank();

class cash
{
	var $currencies = array();
	/**
	 * Get currencies
	 *
	 * @return array available currencies
	 */
	function get_currencies($id = 0, $dropdown = false, $return = 'cash_name')
	{
		global $db;

		if (!sizeof($this->currencies))
		{
			$cashrow = array();

			$sql = 'SELECT *
				FROM ' . CASH_TABLE;
			$result = $db->sql_query($sql);

			while ($row = $db->sql_fetchrow($result))
			{
				$cashrow[$row['cash_id']] = $row;
			}
			$db->sql_freeresult($result);

			$this->currencies = $cashrow;
		}

		if ($dropdown)
		{
			global $user;

			$dropdown = '';
			foreach ($this->currencies as $key => $row)
			{
				$dropdown .= '<option value="' . $key . '"' . (($key == $id) ? 'selected="selected"' : '') . '>' . (isset($user->lang[$row['cash_name']]) ? $user->lang[$row['cash_name']] : $row['cash_name']) . '</option>';
			}

			return $dropdown;
		}

		if ($id)
		{
			if ($return == 'all')
			{
				return $this->currencies[$id];
			}
			return $this->currencies[$id][$return];
		}

		return $this->currencies;
	}

	/**
	 * Format a currency correctly for display
	 *
	 * @param float $amount amount to format
	 * @param int $currency currency to format in
	 * @return string returned formatted currency
	 */
	function format_currency($amount, $currency)
	{
		$cashrow = $this->get_currencies();

		return sprintf($cashrow[$currency]['format'], $amount);
	}

	/**
	 * Get users cash
	 *
	 * @param int/array $user_id can also be passed as an array of user_ids (ints)
	 * @return array users cash in all currencies
	 */
	function get_user_cash($user_id, $total = true)
	{
		global $db, $config;

		$cashrow = array();

		// if we pass an array of users, grab multi users
		if (is_array($user_id))
		{
			$multi = true;
			$where = $db->sql_in_set('ca.user_id', $user_id);
		}
		else
		{
			$multi = false;
			$where = 'ca.user_id = ' . (int) $user_id;
		}

		// set up our total cash amount for each user
		if ($multi)
		{
			$total_cash = array();
		}
		else
		{
			$total_cash = 0;
		}

		// grab all the cash data for this user/users
		$sql = $db->sql_build_query('SELECT', array(
			'SELECT'	=> 'c.*, ca.*',
			'FROM'		=> array(
				CASH_TABLE			=> 'c',
				CASH_AMOUNT_TABLE	=> 'ca',
			),
			'WHERE'		=> $where . ' AND ca.cash_id = c.cash_id',
		));
		$result = $db->sql_query($sql);

		while ($row = $db->sql_fetchrow($result))
		{
			// add the user_id into the mix if we're grabbing for multiple users
			if ($multi)
			{
				$cashrow[$row['user_id']][$row['cash_id']] = $row;
			}
			else
			{
				$cashrow[$row['cash_id']] = $row;
			}

			// if we need the total, add the amount (exchanged with the current conversion rate) to the display currency
			if ($total)
			{
				if ($multi)
				{
					// make sure the total_cash user_id array is set, otherwise create it
					if (isset($total_cash[$row['user_id']]))
					{
						$total_cash[$row['user_id']] += $this->exchange_amounts($row['cash_amt'], $row['cash_id'], $config['cash_display_currency']);
					}
					else
					{
						$total_cash[$row['user_id']] = $this->exchange_amounts($row['cash_amt'], $row['cash_id'], $config['cash_display_currency']);
					}
				}
				else
				{
					$total_cash += $this->exchange_amounts($row['cash_amt'], $row['cash_id'], $config['cash_display_currency']);
				}
			}
		}
		$db->sql_freeresult($result);

		// if we've requested the total, return our total amoutn
		if ($total)
		{
			return $total_cash;
		}

		return $cashrow;
	}

	/**
	 * donate cash
	 *
	 * @param int $from Donation from
	 * @param int $to Donated to
	 * @param float $amount Donation amount
	 * @param int $currency Donation currency
	 * @return bool true if success, false if not successful
	 */
	function donate($from, $to, $amount, $currency)
	{
		global $db;

		if ($amount)
		{
			if (!$this->take_cash($from, $amount, $currency))
			{
				return false;
			}

			$this->give_cash($to, $amount, $currency);

			return true;
		}

		return false;
	}

	/**
	 * Check permissions to manage a users cash on hand
	 *
	 * @param int $user_id
	 * @return unknown true if can manage, string stating reason why if can't manage
	 */
	function check_permissions($user_id)
	{
		global $user, $auth;

		if ($auth->acl_get('m_cash_manage|a_cash_manage'))
		{
			if ($user_id == $user->data['user_id'])
			{
				return 'CANNOT_MANAGE_OWN_CASH';
			}
			else
			{
				return true;
			}
		}
		else
		{
			return 'NO_MANAGE_CASH';
		}
	}

	/**
	 * Give user cash
	 *
	 * @param int $user_id user you're giving cash to
	 * @param float $amount amount to give
	 * @param int $currency currency you're giving the user cash in
	 * @return bool true or false
	 */
	function give_cash($user_id, $amount, $currency)
	{
		global $db;

		if ($amount)
		{
			$sql = 'UPDATE ' . CASH_AMOUNT_TABLE . '
				SET cash_amt = cash_amt + ' . (float) $amount . '
				WHERE cash_id = ' . (int) $currency . '
					AND user_id = ' . $user_id;
			$db->sql_query($sql);

			if (!$db->sql_affectedrows())
			{
				$sql_ary = array(
					'user_id'	=> (int) $user_id,
					'cash_id'	=> (int) $currency,
					'cash_amt'	=> (float) $amount,
				);

				$sql = 'INSERT INTO ' . CASH_AMOUNT_TABLE . ' ' . $db->sql_build_array('INSERT', $sql_ary);
				$db->sql_query($sql);
			}

			return true;
		}

		return false;
	}

	/**
	 * Remove cash from an user
	 *
	 * @param int $user_id id of the user we're removing cash from
	 * @param float $amount amount of cash to take away
	 * @param int $currency currency we're transacting in
	 * @return bool true or false
	 */
	function take_cash($user_id, $amount, $currency)
	{
		global $db;

		if ($amount > 0)
		{
			// check to see if the user has the cash in this currency
			$cashrow = $this->get_user_cash($user_id, false);

			if (isset($cashrow[$currency]['cash_amt']) && $cashrow[$currency]['cash_amt'] >= $amount)
			{
				$sql = 'UPDATE ' . CASH_AMOUNT_TABLE . '
					SET cash_amt = cash_amt - ' . (float) $amount . '
					WHERE user_id = ' . (int) $user_id . '
						AND cash_id = ' . (int) $currency;
				$db->sql_query($sql);

				if ($db->sql_affectedrows())
				{
					return true;
				}
			}
		}

		return false;
	}

	/**
	 * Receive cash for starting new topics
	 *
	 * @param int $forum_id forum_id the topic is being posted in
	 * @param string $message message text
	 * @return int cash received
	 */
	function new_topic($forum_id, $message)
	{
		global $config, $auth, $user;

		$topic_words = $topic_characters = $topic_cash = 0;

		// first, see if this is a forum that this user can receive cash for new topics
		if ($auth->acl_get('u_cash_topic_words', $forum_id))
		{
			$topic_words = $config['cash_topic_per_word'] * str_word_count($message);
		}

		if ($auth->acl_get('u_cash_topic_characters', $forum_id))
		{
			$topic_characters = $config['cash_topic_per_character'] * strlen($message);
		}

		if (!$auth->acl_get('u_cash_topic', $forum_id))
		{
			$topic_cash = $config['cash_topic'];
		}

		$total = $topic_words + $topic_characters + $topic_cash;
		$this->give_cash($user->data['user_id'], $total, $config['cash_topic_currency']);

		return $total;
	}

	/**
	 * Receive cash for posting
	 *
	 * @param int $forum_id forum_id the topic is being posted in
	 * @param string $message message text
	 * @return int cash received
	 */
	function new_post($forum_id, $message)
	{
		global $config, $auth, $user;

		$post_words = $post_characters = $post_cash = 0;

		// first, see if this is a forum that this user can receive cash for new posts
		if ($auth->acl_get('u_cash_post_words', $forum_id))
		{
			$post_words = $config['cash_post_per_word'] * str_word_count($message);
		}

		if ($auth->acl_get('u_cash_post_characters', $forum_id))
		{
			$post_characters = $config['cash_post_per_character'] * strlen($message);
		}

		if (!$auth->acl_get('u_cash_post', $forum_id))
		{
			$post_cash = $config['cash_post'];
		}

		$total = $post_words + $post_characters + $post_cash;
		$this->give_cash($user->data['user_id'], $total, $config['cash_post_currency']);

		return $total;
	}

	/**
	 * Receive cash for creating new polls
	 *
	 * @param int $forum_id forum_id the poll is being created in
	 * @param int $options number of poll options being created
	 * @return int cash received
	 */
	function new_poll($forum_id, $options)
	{
		global $config, $auth, $user;

		$poll_options = $poll_cash = 0;

		// first, see if this is a forum that this user can receive cash for new polls
		if ($auth->acl_get('u_cash_poll_options', $forum_id))
		{
			$poll_options = $config['cash_poll_per_option'] * $options;
		}

		if ($auth->acl_get('u_cash_poll', $forum_id))
		{
			$poll_cash = $config['cash_poll'];
		}

		$total = $poll_options + $poll_cash;
		$this->give_cash($user->data['user_id'], $total, $config['cash_poll_currency']);

		return $total;
	}

	/**
	 * Receive cash for attaching files
	 *
	 * @param int $forum_id forum_id the topic is being posted in
	 * @param int $files number of attached files
	 * @return int cash received
	 */
	function new_attachment($forum_id, $files)
	{
		global $config, $auth, $user;

		$attachment_files = $attachment_cash = 0;

		// first, see if this is a forum that this user can receive cash for new attachments
		if ($auth->acl_get('u_cash_attachment_files', $forum_id))
		{
			$attachment_files = $config['cash_attachment_per_file'] * $files;
		}

		if ($auth->acl_get('u_cash_attachment', $forum_id))
		{
			$attachment_cash = $config['cash_attachment'];
		}

		$total = $attachment_files + $attachment_cash;
		$this->give_cash($user->data['user_id'], $total, $config['cash_attachment_currency']);

		return $total;
	}

	/**
	 * Get equivelant cash amount
	 * This is only used for displaying users cash amounts
	 *
	 * @param float $amount amount to convert
	 * @param int $currency_from currency_id to convert from
	 * @param int $currency_to currency_id to convert to
	 * @return float converted amount
	 */
	function exchange_amounts($amount, $currency_from, $currency_to)
	{
		global $bank;

		// get the exchange rates
		$exchange_rates = $bank->exchange_rates();

		// multiply by it's own exchange rate
		$amount *= $exchange_rates[$currency_from]['exchange_rate'];

		// divide by the new exchange rate
		$amount /= $exchange_rates[$currency_to]['exchange_rate'];

		return $amount;
	}

	/**
	 * add cash to the viewtopic.php user cache
	 *
	 * @param array $id_cache
	 * @param array $user_cache
	 */
	function user_cache($id_cache, &$user_cache)
	{
		if (sizeof($id_cache))
		{
			$cash_cache = $this->get_user_cash($id_cache);

			if (sizeof($cash_cache))
			{
				foreach ($cash_cache as $user => $amount)
				{
					$user_cache[$user]['cash'] = $amount;
				}
			}
		}
	}

	/**
	 * Display cash and donate link in the users profile
	 *
	 * @param int $user_id
	 */
	function viewprofile($user_id)
	{
		global $template, $config, $phpbb_root_path, $phpEx;

		$template->assign_vars(array(
			'CASH_NAME'		=> (isset($user->lang[$config['cash_display_name']]) ? $user->lang[$config['cash_display_name']] : $config['cash_display_name']),
			'CASH'			=> $this->get_user_cash($user_id),

			'U_CASH_DONATE'	=> append_sid($phpbb_root_path . 'ucp.' . $phpEx, 'i=bank&amp;mode=donate&amp;user_id=' . $user_id),
			'U_CASH_MANAGE'	=> '',
		));
	}

	/**
	 * Assign the cash display name to the template
	 *
	 */
	function cash_name()
	{
		global $config, $template, $user;

		$template->assign_var('CASH_NAME', (isset($user->lang[$config['cash_display_name']]) ? $user->lang[$config['cash_display_name']] : $config['cash_display_name']));
	}
}

class bank
{
	/**
	 * Check permission to mangage cash in the bank
	 *
	 * @param int $user_id users account we're trying to manage
	 * @return unknown true if we have permission, string with reason if no permission
	 */
	function check_permissions($user_id)
	{
		global $user, $auth;

		if ($auth->acl_get('m_bank_manager|a_bank_manager'))
		{
			if ($user_id == $user->data['user_id'])
			{
				return 'CANNOT_MANAGE_BANK_ACCOUNT';
			}
			else
			{
				return true;
			}
		}
		else
		{
			return 'NO_BANK_MANAGER';
		}
	}

	/**
	 * Deposit funds into users account
	 *
	 * @param int $user_id
	 * @param float $amount
	 * @param int $currency
	 * @return bool true or false
	 */
	function deposit_cash($user_id, $amount, $currency)
	{
		global $db, $cash;

		if ($amount > 0)
		{
			if (!$cash->take_cash($user_id, $amount, $currency))
			{
				return false;
			}

			$this->transact_deposit($user_id, $amount, $currency);

			return true;
		}

		return false;
	}

	/**
	 * process the deposit transaction
	 *
	 * @param int $user_id user_id to deposit to
	 * @param float $amount amount to deposit
	 * @param int $currency cash_id
	 * @return bool true or false
	 */
	function transact_deposit($user_id, $amount, $currency)
	{
		global $db;

		if ($amount > 0)
		{
			$sql = 'UPDATE ' . BANK_AMT_TABLE . '
				SET cash_amt = cash_amt + ' . (float) $amount . '
				WHERE user_id = ' . (int) $user_id . '
					AND cash_id = ' . (int) $currency;
			$db->sql_query($sql);

			if (!$db->sql_affectedrows())
			{
				$sql_ary = array(
					'user_id'	=> (int) $user_id,
					'cash_id'	=> (int) $currency,
					'cash_amt'	=> $amount,
				);

				$sql = 'INSERT INTO ' . BANK_AMT_TABLE . ' ' . $db->sql_build_array('INSERT', $sql_ary);
				$db->sql_query($sql);
			}

			return true;
		}

		return false;
	}

	/**
	 * Withdrawl funds from users account
	 *
	 * @param int $user_id
	 * @param float $amount
	 * @param int $currency
	 * @return bool true or false
	 */
	function withdrawl_cash($user_id, $amount, $currency)
	{
		global $db, $cash;

		if ($amount > 0)
		{
			// remove it from their bank account first
			if (!$this->transact_withdrawl($user_id, $amount, $currency))
			{
				return false;
			}

			$cash->give_cash($user_id, $amount, $currency);

			return true;
		}

		return false;
	}

	/**
	 * process the deposit transaction
	 *
	 * @param int $user_id user_id to deposit to
	 * @param float $amount amount to deposit
	 * @param int $currency cash_id
	 * @return bool true or false
	 */
	function transact_withdrawl($user_id, $amount, $currency)
	{
		global $db;

		if ($amount > 0)
		{
			$sql = 'UPDATE ' . BANK_AMT_TABLE . '
				SET cash_amt = cash_amt - ' . (float) $amount . '
				WHERE user_id = ' . (int) $user_id . '
					AND cash_id = ' . (int) $currency;
			$db->sql_query($sql);

			if (!$db->sql_affectedrows())
			{
				return false;
			}

			return true;
		}

		return false;
	}

	/**
	 * Check the balance of an users account
	 *
	 * @param int $user_id
	 * @return array balance sorted by cash_id keys
	 */
	function get_balance($user_id)
	{
		global $db;

		$cashrow = array();
		$sql = $db->sql_build_query('SELECT', array(
			'SELECT'	=> 'b.*, c.*',
			'FROM'		=> array(
				BANK_AMT_TABLE	=> 'b',
				CASH_TABLE		=> 'c',
			),
			'WHERE'		=> 'b.user_id = ' . (int) $user_id . '
				AND c.cash_id = b.cash_id',
		));

		$result = $db->sql_query($sql);

		while ($row = $db->sql_fetchrow($result))
		{
			$cashrow[$row['cash_id']] = $row;
		}
		$db->sql_freeresult($result);

		return $cashrow;
	}

	/* Transfers */
	/**
	 * Currency Exchange
	 *
	 * @param int $user_id user_id we're exchanging funds for
	 * @param float $amount amount we're exchanging
	 * @param int $currency_from currency id to exchange from
	 * @param int $currency_to currency id to exchange to
	 * @return bool true or false
	 */
	function exchange_funds($user_id, $amount, $currency_from, $currency_to)
	{
		// take the funds from the users bank account
		if (!$this->transact_withdrawl($user_id, $amount, $currency_from))
		{
			return false;
		}

		// still here? ok, exchange the funds and deposit the new amount minus the exchange rate and the fee
		$exchange_rates = $this->exchange_rates();

		// multiply by it's own exchange rate
		$amount *= $exchange_rates[$currency_from]['exchange_rate'];

		// devide by the new exchange rate
		$amount /= $exchange_rates[$currency_to]['exchange_rate'];

		// pay the new exchange rate fee
		$amount -= $exchange_rates[$currency_to]['exchange_fee'];
		$amount -= $amount * $exchange_rates[$currency_to]['exchange_percent'];

		// now deposit the funds
		$this->transact_deposit($user_id, $amount, $currency_to);

		return true;
	}

	/**
	 * get currency exchange rates
	 *
	 * @return array with cash_id key
	 */
	function exchange_rates()
	{
		global $db;

		$exchangerow = array();
		$sql = 'SELECT *
			FROM ' . CASH_RATES_TABLE;
		$result = $db->sql_query($sql);

		while ($row = $db->sql_fetchrow($result))
		{
			$exchangerow[$row['cash_id']] = $row;
		}
		$db->sql_freeresult($result);

		return $exchangerow;
	}

	/**
	 * Get the current exchange rates
	 *
	 * @return array
	 */
	function get_rates()
	{
		global $db;

		$exchangerow = array();
		$sql = $db->sql_build_query('SELECT', array(
			'SELECT'	=> 'c.*, e.*',
			'FROM'		=> array(
				CASH_TABLE			=> 'c',
				CASH_RATES_TABLE	=> 'e',
			),
			'WHERE'		=> 'c.cash_id = e.cash_id',
		));

		$result = $db->sql_query($sql);

		while ($row = $db->sql_fetchrow($result))
		{
			$exchangerow[$row['cash_id']] = $row;
		}
		$db->sql_freeresult($result);

		return $exchangerow;
	}

	/**
	 * get interest rates
	 *
	 * @return array with cash_ID key
	 */
	function interest_rates()
	{
		global $db;

		$interestrow = array();
		$sql = 'SELECT *
			FROM ' . BANK_INTEREST_TABLE;
		$result = $db->sql_query($sql);

		while ($row = $db->sql_fetchrow($result))
		{
			$interestrow[$row['cash_id']] = $row;
		}
		$db->sql_freeresult($result);

		return $interestrow;
	}
}